$ontext

   Cutting Stock Example Using Column Generation

   Erwin Kalvelagen, december 2002

$offtext


set  i 'widths' /width1*width4/;


*-----------------------------------------------------
* Data
*-----------------------------------------------------

scalar r 'raw width' /100/;

table demanddata(i,*)
           width  demand
  width1    45      97
  width2    36     610
  width3    31     395
  width4    14     211
;


parameter w(i);
w(i) = demanddata(i,'width');

parameter d(i);
d(i) = demanddata(i,'demand');


*-----------------------------------------------------
* Gilmore-Gomory column generation algorithm
*-----------------------------------------------------

set p 'possible patterns' /p1*p1000/;
set iter 'maximum iterations' /iter1*iter25/;


*-----------------------------------------------------
* Master model
*-----------------------------------------------------

parameter aip(i,p) 'matrix growing in dimension p';
integer variable xp(p) 'patterns used';
variable z 'objective variable';

*
* default integer upperbound of 100 is too tight
*
xp.up(p) = sum(i, d(i));

equations
   master_numpat    'number of patterns used'
   master_demand(i) 'meet demand'
;

set pp(p) 'dynamic subset';

master_numpat..     z =e= sum(pp, xp(pp));
master_demand(i)..  sum(pp, aip(i,pp)*xp(pp)) =g= d(i);
model master /master_numpat,master_demand/;

* reduce amount of information written to the listing file
master.solprint = 2;
master.limrow = 0;
master.limcol = 0;
* faster execution of solve statements: keep gams in memory
master.solvelink = 2;


*-----------------------------------------------------
* Knapsack model
*-----------------------------------------------------

integer variables
   y(i)  'new pattern'
;
y.up(i) = ceil(r/w(i));

equations
   knapsack_obj
   knapsack_constraint
;

knapsack_obj..         z =e= 1 - sum(i, master_demand.m(i)*y(i));
knapsack_constraint..  sum(i, w(i)*y(i)) =l= r;
model knapsack /knapsack_obj,knapsack_constraint/;

knapsack.solprint = 2;
knapsack.solvelink = 2;
knapsack.optcr = 0;
knapsack.limrow = 0;
knapsack.limcol = 0;


*-----------------------------------------------------
* initialization
* get initial set pp and initial matrix aip
*-----------------------------------------------------
set pi(p);
pi('p1') = yes;
loop(i,
    aip(i,pi) = floor(r/w(i));
    pp(pi) = yes;
    pi(p) = pi(p-1);
);
display "--------------------------------------------------------------",
        "Initial value",
        "--------------------------------------------------------------",
        pp,aip;


scalar done /0/;
scalar iteration;

loop(iter$(not done),

*
* solve master problem
*
   solve master using rmip minimizing z;

*
* solve knapsack problem
*
   solve knapsack using mip minimizing z;

*
* new pattern found?
*
   if(z.l < -0.001,
      aip(i,pi) = y.l(i);
      pp(pi) = yes;
      iteration = ord(iter);
      display "--------------------------------------------------------------",
              iteration,
              "--------------------------------------------------------------",
              pp,aip;
      pi(p) = pi(p-1);
   else
      done = 1;
   );
);

abort$(not done) "Too many iterations.";


*
* solve final mip
*

display "--------------------------------------------------------------",
        "Final MIP",
        "--------------------------------------------------------------";

master.optcr=0;
solve master using mip minimizing z;
display z.l,xp.l;

parameter pat(*,*) "pattern usage";
pat(i,p)$(xp.l(p)>0.1) = aip(i,p);
pat('Count',p) = round(xp.l(p));
pat(i,'Total')= sum(p, aip(i,p)*round(xp.l(p)));
pat('Count','Total') = sum(p,pat('Count',p));
display pat;